// Web.cpp
/*
Copyright (C) 2003  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#ifdef WIN32
#include <winsock2.h>
#else
#include <sys/time.h>
#include <unistd.h>
#endif
#include <string>
using std::string;

#include "Form.h"
#include "Cookies.h"
//#include "WebForm.h"

#include <vector>
#include "WebForm.h"
#include "Web.h"

#ifdef WIN32
#define strcasecmp stricmp
#endif


Web::Web()
:m_pclForm(NULL)
,m_pclCookies(NULL)
,m_strCookieDomain("")
,m_strCookiePath("")
{
	time_t t = time(NULL);
	struct tm * tp = localtime(&t);
	char date[40];

	sprintf(date,"%d-%02d-%02d %02d:%02d:%02d",
		tp -> tm_year + 1900,
		tp -> tm_mon + 1,
		tp -> tm_mday,
		tp -> tm_hour,
		tp -> tm_min,
		tp -> tm_sec);
	m_date = date;

//	Init();
}

Web::Web(const string &strd,const string &strp)
:m_pclForm(NULL)
,m_pclCookies(NULL)
,m_strCookieDomain(strd)
,m_strCookiePath(strp)
{
	time_t t = time(NULL);
	struct tm * tp = localtime(&t);
	char date[40];

	sprintf(date,"%d-%02d-%02d %02d:%02d:%02d",
		tp -> tm_year + 1900,
		tp -> tm_mon + 1,
		tp -> tm_mday,
		tp -> tm_hour,
		tp -> tm_min,
		tp -> tm_sec);
	m_date = date;

	Init();
}


void Web::RegWebForm(WebForm *pclForm)
{
	m_forms.insert(m_forms.end(), pclForm);
}


void Web::Init()
{
	char *r_m = getenv("REQUEST_METHOD");
//	char *c_l = getenv("CONTENT_LENGTH");
	char *q_s = getenv("QUERY_STRING");
//	char *r_a = getenv("REMOTE_ADDR");
	char *h_c = getenv("HTTP_COOKIE");

#ifdef WIN32
#else
	struct timeval tv;

	gettimeofday(&tv,NULL);
	srandom( (tv.tv_sec % 1000000) * 1000 + tv.tv_usec);
#endif

	if (!r_m)
	{
		fprintf(stderr,"This is a cgi program\n");
		exit(-1);
	}
	m_method = r_m;
	if (!strcasecmp(r_m,"get"))
	{
		if (q_s)
		{
			m_pclForm = new Form(q_s,strlen(q_s));
		}
		else
		{
#ifdef WIN32
			// IIS won't set QUERY_STRING if it's empty
			m_pclForm = new Form("",0);
#else
			fprintf(stderr,"QUERY_STRING undefined\n");
			exit(-1);
#endif
		}
	}
	else
	if (!strcasecmp(r_m,"post"))
	{
		m_pclForm = new Form;
	}
	else
	{
		fprintf(stderr,"Unknown request method: '%s'\n",r_m);
		exit(-1);
	}
	// Retrieve cookies
	m_pclCookies = new Cookies(h_c ? h_c : (char *)"");
}


Web::~Web()
{
	if (m_pclForm)
	{
		delete m_pclForm;
	}
	if (m_pclCookies)
	{
		delete m_pclCookies;
	}
}


void Web::Execute()
{
	ReadCookies();
	FormInput();
	CreateHeader();
	GenerateDocument();
}


void Web::ReadCookies()
{
}


void Web::FormInput()
{
	std::vector<WebForm *>::iterator it;
	
	for (it = m_forms.begin(); it != m_forms.end(); it++)
	{
		WebForm *pclForm = static_cast<WebForm *>(*it);
		pclForm -> Process();
	}
}


void Web::CreateHeader()
{
	// character set
	printf("Content-type: text/html; charset=ISO-8859-1\n");
	printf("\n");
	fflush(stdout);

	// 4.01 strict
//	printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">\n");
	// 4.01 transitional
//	printf("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd\">\n");
	fflush(stdout);
}


void Web::GenerateDocument()
{
}


void Web::SetCookieDomain(const string &str)
{
	m_strCookieDomain = str;
}


void Web::SetCookiePath(const string &str)
{
	m_strCookiePath = str;
}


void Web::SetCookie(const string &name,const string &value)
{
	m_pclCookies -> setcookie( (char *)m_strCookieDomain.c_str(),
		(char *)m_strCookiePath.c_str(),
		(char *)name.c_str(),
		(char *)value.c_str());
}


void Web::SetCookie(const string &name,long value)
{
	m_pclCookies -> setcookie( (char *)m_strCookieDomain.c_str(),
		(char *)m_strCookiePath.c_str(),
		(char *)name.c_str(),
		value);
}


void Web::SetCookie(const string &name,int value)
{
	m_pclCookies -> setcookie( (char *)m_strCookieDomain.c_str(),
		(char *)m_strCookiePath.c_str(),
		(char *)name.c_str(),
		value);
}


void Web::SetCgiName(const string &name)
{
	m_strCgiName = name;
}


string Web::GetCgiName()
{
	return m_strCgiName;
}


Form *Web::GetForm()
{
	return m_pclForm;
}


Cookies *Web::GetCookies()
{
	return m_pclCookies;
}


string Web::GetDate() 
{ 
	return m_date; 
}


string Web::GetRequestMethod() 
{ 
	return m_method; 
}


